@@ -135,7 +135,8 @@ int main(int argc, char* argv[])
 
 #if defined(HAVE_GC_H) || defined(HAVE_GC_GC_H)
     /// GC 開始 ///
-    GC_init();
+    GC_INIT();
+//GC_enable_incremental();
 #endif
 
     /// 乱数初期化 ///
