@@ -576,6 +576,7 @@ char* readline_file_completion_migemo(const char* text, int rstat)
                         }
 
                         vector_add(files, STRING_NEW(buf));
+
                     }
                 }
 
@@ -1004,6 +1005,87 @@ char* readline_object_completion(const char* text, int stat)
     return NULL;
 }
 
+typedef struct {
+    regex_t* mRegex;
+    sStatments* mStatments;
+} sUserRegex;
+
+sUserRegex* gUserCompRegex[1024];
+int gUserCompRegexNum = 0;
+
+sUserRegex* sUserRegex_new(regex_t* regex, sStatments* statments)
+{
+    sUserRegex* self = MALLOC(sizeof(sUserRegex));
+
+    self->mRegex = regex;
+    self->mStatments = statments;
+
+    return self;
+}
+
+static vector_obj* gUserCompletions;
+
+static string_obj* gUserCompletionStatment;
+static sStatments* gUserCompletionStatments;
+
+char* readline_user_completion(const char* text, int stat)
+{
+    static int index, wordlen;
+
+    if(stat == 0) {
+        wordlen = strlen(text);
+        index = 0;
+
+        vector_clear(gUserCompletions);
+
+        sWFd* pipeout = sWFd_new(-1);
+        sRFd* pipein = sRFd_new2(-1, string_c_str(gUserCompletionStatment));
+        int rcode = saphire_run(gUserCompletionStatments, "user completion", pipeout, pipein,  2);
+        if(rcode >= 0) {
+            char* p = pipeout->mBuffer;
+            while(1) {
+                string_obj* str = STRING_NEW("");
+                while(1) {
+                    if(*p == 0) {
+                        break;
+                    }
+                    else if(*p == '\n') {
+                        p++;
+                        break;
+                    }
+                    else {
+                        string_push_back2(str, *p);
+                        p++;
+                    }
+                }
+
+                if(string_c_str(str)[0] != 0) {
+                    vector_add(gUserCompletions, str);
+                }
+
+                if(*p == 0) break;
+            }
+        }
+        else {
+            fprintf(stderr, "run time err\n");
+            fprintf(stderr, "%s", string_c_str(gErrMsg));
+
+            return NULL;
+        }
+    }
+
+    while(index < vector_size(gUserCompletions)) {
+        char* ucomp = string_c_str(vector_item(gUserCompletions, index));
+        index++;
+
+        if(!strncmp(text, ucomp, wordlen)) {
+            return strdup(ucomp);
+        }
+    }
+
+    return NULL;
+}
+
 BOOL gDerefCompletionProgram = FALSE;
 
 char* readline_deref_completion(const char* text, int stat)
@@ -1062,8 +1144,10 @@ char* readline_ref_completion(const char* text, int stat)
     return NULL;
 }
 
+
 char* readline_file_completion(const char* text, int rstat)
 {
+
     /// クォートを除外 ///
     char* text2 = MALLOC(strlen(text) +1);
     char* _p = (char*)text;
@@ -1281,7 +1365,6 @@ char* readline_file_completion(const char* text, int rstat)
                 string_obj* tmp2 = STRING_NEW("");
                 saphire_get_quoted_fname(path, tmp2);
                 char* ret = strdup(string_c_str(tmp2));
-
                 xstrncpy(path, string_c_str(gEditingDir), PATH_MAX);
                 xstrncat(path, file, PATH_MAX);
                 struct stat stat_;
@@ -1311,6 +1394,28 @@ char* readline_file_completion(const char* text, int rstat)
     return NULL;
 }
 
+static regex_t* get_reg(int* r, char* regex2)
+{
+    regex_t* reg;
+    OnigEncoding enc = ONIG_ENCODING_UTF8;
+    OnigErrorInfo err_info;
+    *r = onig_new(&reg, regex2
+            , regex2 + strlen((char*)regex2)
+            , ONIG_OPTION_DEFAULT
+            , enc
+            //, ONIG_SYNTAX_RUBY
+            //, ONIG_SYNTAX_PERL_NG
+            , ONIG_SYNTAX_DEFAULT
+            , &err_info);
+
+    if(*r != ONIG_NORMAL) {
+        onig_free(reg);
+    }
+
+    return reg;
+}
+
+
 char** readline_on_complete(const char* text, int start, int end)
 {
     BOOL program = TRUE;
@@ -1318,6 +1423,7 @@ char** readline_on_complete(const char* text, int start, int end)
     BOOL ref = FALSE;
     BOOL method = FALSE;
     string_obj* command = STRING_NEW("");
+    string_obj* statment = STRING_NEW("");
 
     string_obj* buf = STRING_NEW("");
 
@@ -1332,28 +1438,51 @@ char** readline_on_complete(const char* text, int start, int end)
     while(p < rl_line_buffer + end) {
         // クォート
         if(*p == '\\' && *(p+1) == 't') {
-            p+=2;
+            string_push_back2(statment, *p);
+            p++;
+            string_push_back2(statment, *p);
+            p++;
             user = FALSE;
             ref = FALSE;
 
             string_put(buf, "");
         }
         else if(*p == '\\' && *(p+1) == 'n') {
-            p+=2;
+            string_push_back2(statment, *p);
+            p++;
+            string_push_back2(statment, *p);
+            p++;
             user = FALSE;
             ref = FALSE;
 
             string_put(buf, "");
         }
         else if(*p == '\\' && *(p+1) == 'r') {
-            p+=2;
+            string_push_back2(statment, *p);
+            p++;
+            string_push_back2(statment, *p);
+            p++;
+            user = FALSE;
+            ref = FALSE;
+
+            string_put(buf, "");
+        }
+        else if(*p == '\\' && *(p+1) == 'a') {
+            string_push_back2(statment, *p);
+            p++;
+            string_push_back2(statment, *p);
+            p++;
             user = FALSE;
             ref = FALSE;
 
             string_put(buf, "");
         }
         else if(*p == '\\') {
-            p+=2;
+            string_push_back2(statment, *p);
+            p++;
+            if(program) { string_push_back2(command, *p); }
+            string_push_back2(statment, *p);
+            p++;
             user = FALSE;
             ref = FALSE;
 
@@ -1362,6 +1491,7 @@ char** readline_on_complete(const char* text, int start, int end)
 
         // シングルクォート
         else if(!dquote && *p == '\'') {
+            string_push_back2(statment, *p);
             p++;
             squote = !squote;
             user = FALSE;
@@ -1370,6 +1500,7 @@ char** readline_on_complete(const char* text, int start, int end)
         }
         // ダブルクォート
         else if(!squote && *p == '"') {
+            string_push_back2(statment, *p);
             p++;
             dquote = !dquote;
             user = FALSE;
@@ -1383,6 +1514,7 @@ char** readline_on_complete(const char* text, int start, int end)
 
             program = TRUE;
             string_put(command, "");
+            string_put(statment, "");
 
             while(*p == ' ' || *p == '\t' || *p == '\n') {
                 p++;
@@ -1398,6 +1530,7 @@ char** readline_on_complete(const char* text, int start, int end)
 
             program = TRUE;
             string_put(command, "");
+            string_put(statment, "");
             while(*p == ' ' || *p == '\t' || *p == '\n') {
                 p++;
             }
@@ -1416,10 +1549,13 @@ char** readline_on_complete(const char* text, int start, int end)
             ref = FALSE;
 
             string_put(buf, "");
+            string_put(statment, "");
+            string_put(command, "");
         }
 
         /// シングルクォート、ダブルクォート中 ///
         else if(squote || dquote) {
+            string_push_back2(statment, *p);
             p++;
 
             string_put(buf, "");
@@ -1427,10 +1563,14 @@ char** readline_on_complete(const char* text, int start, int end)
         // リファレンス
         else if(*p == '$') {
             string_push_back2(buf, *p);
+            if(program) { string_push_back2(command, *p); }
+            string_push_back2(statment, *p);
             p++;
             ref = TRUE;
         }
         else if(*p == '~') {
+            if(program) { string_push_back2(command, *p); }
+            string_push_back2(statment, *p);
             p++;
             ref = FALSE;
             user = TRUE;
@@ -1440,6 +1580,8 @@ char** readline_on_complete(const char* text, int start, int end)
         /// グロブ ///
         else if(*p == '*' || *p == '?' || *p =='[' && (*(p+1) != ' ' && *(p+1) != '\t' && *(p+1) == '\n')) 
         {
+            if(program) { string_push_back2(command, *p); }
+            string_push_back2(statment, *p);
             p++;
             ref = FALSE;
             user = FALSE;
@@ -1489,6 +1631,7 @@ char** readline_on_complete(const char* text, int start, int end)
         /// 空白 ///
         else if(*p == ' ' || *p == '\t') {
             while(*p == ' ' || *p == '\t') {
+                string_push_back2(statment, *p);
                 p++;
             }
 
@@ -1504,6 +1647,7 @@ char** readline_on_complete(const char* text, int start, int end)
             p++;
             program = TRUE;
             string_put(command, "");
+            string_put(statment, "");
             method = FALSE;
             while(*p == ' ' || *p == '\t' || *p == '\n') {
                 p++;
@@ -1516,30 +1660,16 @@ char** readline_on_complete(const char* text, int start, int end)
         
         /// コマンドの区切り ///////////////////////////////////////////
         
-        /// パイプ ///
-        else if(*p == '|' && *(p+1) == '~') {
-            p+=2;
-
-            program = TRUE;
-            string_put(command, "");
-            method = FALSE;
-            while(*p == ' ' || *p == '\t' || *p == '\n') {
-                p++;
-            }
-            ref = FALSE;
-            user = FALSE;
-
-            string_put(buf, "");
-        }
-        
         /// パイプ ///
         else if(*p == '|' && *(p+1) != '|') {
+            string_push_back2(statment, *p);
             p++;
 
             program = TRUE;
             string_put(command, "");
             method = FALSE;
             while(*p == ' ' || *p == '\t' || *p == '\n') {
+                string_push_back2(statment, *p);
                 p++;
             }
             ref = FALSE;
@@ -1556,8 +1686,10 @@ char** readline_on_complete(const char* text, int start, int end)
 
             program = TRUE;
             string_put(command, "");
+            string_put(statment, "");
             method = FALSE;
             while(*p == ' ' || *p == '\t' || *p == '\n') {
+                string_push_back2(statment, *p);
                 p++;
             }
             user = FALSE;
@@ -1571,8 +1703,10 @@ char** readline_on_complete(const char* text, int start, int end)
 
             program = TRUE;
             string_put(command, "");
+            string_put(statment, "");
             method = FALSE;
             while(*p == ' ' || *p == '\t' || *p == '\n') {
+                string_push_back2(statment, *p);
                 p++;
             }
             user = FALSE;
@@ -1586,8 +1720,10 @@ char** readline_on_complete(const char* text, int start, int end)
 
             program = TRUE;
             string_put(command, "");
+            string_put(statment, "");
             method = FALSE;
             while(*p == ' ' || *p == '\t' || *p == '\n') {
+                string_push_back2(statment, *p);
                 p++;
             }
             user = FALSE;
@@ -1601,8 +1737,10 @@ char** readline_on_complete(const char* text, int start, int end)
 
             program = TRUE;
             string_put(command, "");
+            string_put(statment, "");
             method = FALSE;
             while(*p == ' ' || *p == '\t' || *p == '\n') {
+                string_push_back2(statment, *p);
                 p++;
             }
             user = FALSE;
@@ -1616,8 +1754,10 @@ char** readline_on_complete(const char* text, int start, int end)
 
             program = TRUE;
             string_put(command, "");
+            string_put(statment, "");
             method = FALSE;
             while(*p == ' ' || *p == '\t' || *p == '\n') {
+                string_push_back2(statment, *p);
                 p++;
             }
             user = FALSE;
@@ -1654,6 +1794,8 @@ char** readline_on_complete(const char* text, int start, int end)
             program = FALSE;
             method = FALSE;
             ref = FALSE;
+            if(program) { string_push_back2(command, *p); }
+            string_push_back2(statment, *p);
             p++;
 
             string_put(buf, "");
@@ -1663,17 +1805,19 @@ char** readline_on_complete(const char* text, int start, int end)
             break;
         }
         
-        /// 構文 ////////////////////////////////////////////////////////
+        /// 構文 ////////////////////////////////////////////////////
 
         /// サブシェル ///
         else if(*p == '(') {
             p++;
             program = TRUE;
             string_put(command, "");
+            string_put(statment, "");
             method = FALSE;
             user = FALSE;
             ref = FALSE;
             while(*p == ' ' || *p == '\t' || *p == '\n') {
+                string_push_back2(statment, *p);
                 p++;
             }
 
@@ -1685,10 +1829,41 @@ char** readline_on_complete(const char* text, int start, int end)
             if(program) {
                 string_push_back2(command, *p);
             }
+            string_push_back2(statment, *p);
             p++;
         }
     }
 
+    sStatments* statments = NULL;
+    int i;
+    for(i=0; i< 1024; i++) {
+        sUserRegex* ureg = gUserCompRegex[i];
+
+        if(ureg) {
+            regex_t* reg = ureg->mRegex;
+            char* str2 = string_c_str(command);
+
+            OnigRegion* region = onig_region_new();
+            int r2 = onig_search(reg, str2
+               , str2 + strlen(str2)
+               , str2, str2 + strlen(str2)
+               , region, ONIG_OPTION_NONE);
+            onig_region_free(region, 1);
+
+            if(r2 >= 0) {
+                statments = ureg->mStatments;
+                break;
+            }
+        }
+    }
+
+/*
+    if(statments) {
+        gUserCompletionStatment = statment;
+        gUserCompletionStatments = sStatments_new2(statments);
+        return rl_completion_matches(text, readline_user_completion);
+    }
+*/
     if(user) {
         rl_completion_append_character= 0;
         return rl_completion_matches(text, rl_username_completion_function);
@@ -1707,67 +1882,11 @@ char** readline_on_complete(const char* text, int start, int end)
     }
 #if defined(HAVE_MIGEMO_H)
     else {
-        if(strcmp(string_c_str(command), "sweep") == 0) {
-            return rl_completion_matches(text, readline_sweep_completion);
-        }
-        else if(strcmp(string_c_str(command), "ary") == 0 || strcmp(string_c_str(command), "ary_add") == 0 || strcmp(string_c_str(command), "ary_erase") == 0)
-        {
-            return rl_completion_matches(text, readline_ary_completion);
-        }
-        else if(strcmp(string_c_str(command), "hash") == 0 || strcmp(string_c_str(command), "hash_add") == 0 || strcmp(string_c_str(command), "hash_erase") == 0)
-        {
-            return rl_completion_matches(text, readline_hash_completion);
-        }
-        else if(strcmp(string_c_str(command), "global") == 0) {
-            return rl_completion_matches(text, readline_global_completion);
-        }
-        else if(strcmp(string_c_str(command), "class") == 0) {
-            return rl_completion_matches(text, readline_class_completion);
-        }
-        else if(strcmp(string_c_str(command), "def") == 0) {
-            return rl_completion_matches(text, readline_def_completion);
-        }
-        else if(strcmp(string_c_str(command), "object") == 0) {
-            return rl_completion_matches(text, readline_object_completion);
-        }
-        else if(strcmp(string_c_str(command), "ref") == 0) {
-            return rl_completion_matches(text, readline_ref_completion);
-        }
-        else {
-            return rl_completion_matches(text, readline_file_completion_migemo);
-        }
+        return rl_completion_matches(text, readline_file_completion_migemo);
     }
 #else
     else {
-        if(strcmp(string_c_str(command), "sweep") == 0) {
-            return rl_completion_matches(text, readline_sweep_completion);
-        }
-        else if(strcmp(string_c_str(command), "ary") == 0 || strcmp(string_c_str(command), "ary_add") == 0 || strcmp(string_c_str(command), "ary_erase") == 0)
-        {
-            return rl_completion_matches(text, readline_ary_completion);
-        }
-        else if(strcmp(string_c_str(command), "hash") == 0 || strcmp(string_c_str(command), "hash_add") == 0 || strcmp(string_c_str(command), "hash_erase") == 0)
-        {
-            return rl_completion_matches(text, readline_hash_completion);
-        }
-        else if(strcmp(string_c_str(command), "global") == 0) {
-            return rl_completion_matches(text, readline_global_completion);
-        }
-        else if(strcmp(string_c_str(command), "class") == 0) {
-            return rl_completion_matches(text, readline_class_completion);
-        }
-        else if(strcmp(string_c_str(command), "def") == 0) {
-            return rl_completion_matches(text, readline_def_completion);
-        }
-        else if(strcmp(string_c_str(command), "object") == 0) {
-            return rl_completion_matches(text, readline_object_completion);
-        }
-        else if(strcmp(string_c_str(command), "ref") == 0) {
-            return rl_completion_matches(text, readline_ref_completion);
-        }
-        else {
-            return rl_completion_matches(text, readline_file_completion);
-        }
+        return rl_completion_matches(text, readline_file_completion);
     }
 #endif
 
@@ -1939,8 +2058,85 @@ BOOL commands_remove_history(int* rcode, vector_obj* argv, vector_obj* blocks, v
     return TRUE;
 }
 
+
+BOOL commands_user_completion(int* rcode, vector_obj* argv, vector_obj* blocks, vector_obj* parent_blocks, sWFd* nextout, sRFd* nextin, int nexterr, char* title, BOOL input, char* sname, int sline)
+{
+    if(vector_size(argv) >= 2 && vector_size(blocks) == 1) {
+        int i;
+        for(i=1; i<vector_size(argv); i++) {
+            char* regex = string_c_str(vector_item(argv, i));
+            sBlock* block = vector_item(blocks, 0);
+            sStatments* statments = sStatments_new2(block->mStatments);
+
+            int r;
+            regex_t* reg = get_reg(&r, regex);
+            if(r == ONIG_NORMAL) {
+                gUserCompRegex[gUserCompRegexNum++] = sUserRegex_new(reg, statments);
+                *rcode = 0;
+            }
+            else {
+                err_msg("invalid regex", sname, sline);
+                return FALSE;
+            }
+        }
+    }
+
+    return TRUE;
+}
+
+static void read_rc_obj_file(BOOL run_user_runtimescript)
+{
+    char rc_fname[PATH_MAX];
+    snprintf(rc_fname, PATH_MAX, "%s/user_compl.sao", SYSCONFDIR);
+
+    if(access(rc_fname, R_OK) == 0) {
+        sWFd* pipeout = sWFd_new(STDOUT_FILENO);
+        sRFd* pipein = sRFd_new(STDIN_FILENO);
+        saphire_init_stack_frame();
+        if(saphire_load_obj(rc_fname
+           , pipeout, pipein, STDERR_FILENO, NULL) < 0)
+        {
+            fprintf(stderr, "%s", string_c_str(gErrMsg));
+            exit(1);
+        }
+        if(!sWFd_flash(pipeout)) {
+            fprintf(stderr, "can't flash rc");
+            exit(1);
+        }
+    }
+    else {
+        fprintf(stderr, "can't find %s file\n", rc_fname);
+        exit(1);
+    }
+
+    /// ユーザーのランタイムスクリプト ////
+    if(run_user_runtimescript) {
+        char* user_rc_fname = getenv("SAPHIRE_COMPL_USERRC");
+        if(user_rc_fname) {
+            if(access(user_rc_fname, R_OK) == 0) {
+                sWFd* pipeout = sWFd_new(STDOUT_FILENO);
+                sRFd* pipein = sRFd_new(STDIN_FILENO);
+                saphire_init_stack_frame();
+                if(saphire_load_obj(user_rc_fname
+                    , pipeout, pipein, STDERR_FILENO, NULL) < 0)
+                {
+                    fprintf(stderr, "%s", string_c_str(gErrMsg));
+                    exit(1);
+                }
+                sWFd_flash(pipeout);
+            }
+        }
+    }
+}
+
 void readline_init()
 {
+    memset(gUserCompRegex, 0, sizeof(gUserCompRegex));
+    gUserCompletions = VECTOR_NEW(50);
+
+    gUserCompletionStatment = NULL;
+    gUserCompletionStatments = NULL;
+
     rl_attempted_completion_function = readline_on_complete;
     rl_completer_quote_characters = "\"'";
     rl_completer_word_break_characters = " \t\n\"'|{}!&;()<>=%/";
@@ -1964,6 +2160,8 @@ void readline_init()
     saphire_add_inner_command("write_history", commands_write_history);
     saphire_add_inner_command("clear_history", commands_clear_history);
     saphire_add_inner_command("remove_history", commands_remove_history);
+    saphire_add_inner_command("user_completion", commands_user_completion);
+    //read_rc_obj_file(TRUE);
 }
 
 void readline_final()
