@@ -262,7 +262,7 @@ extern void saphire_get_quoted_fname(char* fname, string_obj* quoted_fname);
         // saphireシェルで使われる特殊な文字をクォートした文字列を返す
         // &,|,>など
 
-typedef BOOL (*fInnerCommand)(int* rcode, vector_obj* argv, vector_obj* blocks, vector_obj* parent_blocks, sWFd* nextout, sRFd* nextin, int nexterr, char* title, BOOL input);
+typedef BOOL (*fInnerCommand)(int* rcode, vector_obj* argv, vector_obj* blocks, vector_obj* parent_blocks, sWFd* nextout, sRFd* nextin, int nexterr, char* title, BOOL input, char* sname, int sline);
         // 内部コマンドの実行コード
         // 戻り値は実行に成功したらTRUE, 実行に失敗したらFALSEを返してください。
         // rcodeはリターンコード. 何も入力しなかったら最初は1が入っている
@@ -283,6 +283,7 @@ typedef BOOL (*fInnerCommand)(int* rcode, vector_obj* argv, vector_obj* blocks,
         // ls | command などのcommand
         // | command (コンテキストパイプ)とかのcommandの場合はTRUE
         // command | lessのcommandだとFALSE
+        // sname,sline ソースファイルの位置
 
 void saphire_add_inner_command(char* name, fInnerCommand fun);
        // 内部コマンドの追加。上のfInnerCommandを実装して第二引数に
