@@ -1517,6 +1517,9 @@ sStatment* sStatment_new()
     self->mGlobalPipeIn = FALSE;
     self->mGlobalPipeOut = FALSE;
 
+    self->mGlobalPipeInNum = 0;
+    self->mGlobalPipeOutNum = 0;
+
     self->mSTDINPipe = FALSE;
 
     return self;
@@ -1549,6 +1552,9 @@ sStatment* sStatment_new2(sStatment* statment)
     self->mGlobalPipeIn = statment->mGlobalPipeIn;
     self->mGlobalPipeOut = statment->mGlobalPipeOut;
 
+    self->mGlobalPipeInNum = statment->mGlobalPipeInNum;
+    self->mGlobalPipeOutNum = statment->mGlobalPipeOutNum;
+
     self->mSTDINPipe = statment->mSTDINPipe;
 
     return self;
@@ -1848,6 +1854,7 @@ sStatments* sStatments_new2(sStatments* statments)
 
     return self;
 }
+
 void sStatments_view(sStatments* self)
 {
 
@@ -2542,7 +2549,7 @@ void saphire_init(enum eAppType app_type, BOOL job_control, enum eRuntimeScript
 
     gReadBuffers = HASH_NEW(100);
 
-    setenv("SAPHIRE_VERSION", "3.5.0", 1);
+    setenv("SAPHIRE_VERSION", "3.5.1", 1);
     setenv("SAPHIRE_PROMPT", "> ", 1);
     setenv("SAPHIRE_DOCDIR", DOCDIR, 1);
     char* home = getenv("HOME");
