@@ -563,7 +563,7 @@ static BOOL read_one_argument(char** p, string_obj* buf, string_obj* not_evaled,
             (*p)++;
         }
         // クィックコマンド展開は下で処理
-        else if(**p == '@' && *(*p+1) == '(' || **p == '@' && *(*p+1) == '@' && *(*p+2) == '(') {
+        else if((**p == '@' && *(*p+1) == '(') || (**p == '@' && *(*p+1) == '@' && *(*p+2) == '(') || (**p == '@' && *(*p+1) == '@' && *(*p+2) >= 'a' && *(*p+2) <= 'z' && *(*p+3) == '(')) {
             break;
         }
         // % のりダイレクトは下で処理
@@ -907,19 +907,46 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, strin
         }
         
         /// クィックコマンド展開 ///
-        if((**p == '@' && *(*p+1) == '(') || (**p == '@' && *(*p+1) == '@' && *(*p+2) == '(')) {
+        if((**p == '@' && *(*p+1) == '(') || (**p == '@' && *(*p+1) == '@' && *(*p+2) == '(') || (**p == '@' && *(*p+1) == '@' && *(*p+2) >= 'a' && *(*p+2) <= 'z' && *(*p+3) == '(')) 
+            {
+            string_push_back2(not_evaled, **p);
+            (*p)++;
+
             BOOL double_;
-            if(*(*p+1) == '@') {
-                double_ = TRUE;
+            enum eLineField lf;
+            if(**p == '@') {
                 string_push_back2(not_evaled, **p);
                 (*p)++;
-            }
-            else {
-                double_ = FALSE;
-            }
 
-            string_push_back2(not_evaled, **p); // @
-            (*p)++;
+                double_ = TRUE;
+                if(**p == '(') {
+                    lf = kLF;
+                }
+                else if(*(*p+1) == 'a') {
+                    lf = kBel;
+                    string_push_back2(not_evaled, **p);
+                    (*p)++;
+                }
+                else if(*(*p+1) == 'm') {
+                    lf = kCR;
+                    string_push_back2(not_evaled, **p);
+                    (*p)++;
+                }
+                else if(*(*p+1) == 'w') {
+                    lf = kCRLF;
+                    string_push_back2(not_evaled, **p);
+                    (*p)++;
+                }
+                else {
+                    lf = kLF;
+                    string_push_back2(not_evaled, **p);
+                    (*p)++;
+                }
+            }
+            else if(**p == '(') {
+                double_ = FALSE;
+                lf = kLF;
+            }
 
             string_obj* str = STRING_NEW("");
             int read_end_of_statment = -1;
@@ -939,7 +966,7 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, strin
             }
 
             if(double_) {
-                vector_add(command->mArgs, sArg_new(4, statments));
+                vector_add(command->mArgs, sArg_new(4, sAtCommand_new(statments, lf)));
             }
             else {
                 vector_add(command->mArgs, sArg_new(2, statments));
