@@ -19,6 +19,7 @@ string_obj* string_new_debug(char* str, const char* fname, int line, const char*
     
     self->mMallocLen = (len < 128) ? 128 : (len + 1) * 2;
     self->mStr = (char*)MALLOC(sizeof(char)*self->mMallocLen);
+if(self->mStr == NULL) { puts("!!!!!!!!!!!!!!!!!!!!!!!!!!"); }
     xstrncpy(self->mStr, str, self->mMallocLen);
     
     self->mLen = len;
@@ -32,7 +33,7 @@ string_obj* string_new(char* str)
     
     string_obj* self = (string_obj*)MALLOC(sizeof(string_obj));
     
-    self->mMallocLen = (len < 128) ? 128 : (len + 1) * 2;
+    self->mMallocLen = (len < 127) ? 128 : (len + 1) * 2;
     self->mStr = (char*)MALLOC(sizeof(char)*self->mMallocLen);
     xstrncpy(self->mStr, str, self->mMallocLen);
     
@@ -113,6 +114,7 @@ void string_push_back(string_obj* self, char* str)
         FREE(self->mStr);
 
         self->mStr = new_str;
+if(self->mStr == NULL) { puts("!!!!!!!!!!!!!!!!!!!!!!!!!!"); }
         self->mLen = self->mLen + str_len;
     }
 }
@@ -140,6 +142,7 @@ void string_push_back2(string_obj* self, char c)
         FREE(self->mStr);
 
         self->mStr = new_str;
+if(self->mStr == NULL) { puts("!!!!!!!!!!!!!!!!!!!!!!!!!!"); }
         self->mLen++;
     }
 }
@@ -170,6 +173,7 @@ void string_push_back3(string_obj* self, char* str, int n)
         FREE(self->mStr);
 
         self->mStr = new_str;
+if(self->mStr == NULL) { puts("!!!!!!!!!!!!!!!!!!!!!!!!!!"); }
         self->mLen = strlen(new_str);
     }
     else {
@@ -212,6 +216,7 @@ void string_put(string_obj* self, char* str)
         
         self->mMallocLen = (len < 16) ? 16 : (len + 1) * 2;
         self->mStr = (char*)MALLOC(sizeof(char)*self->mMallocLen);
+if(self->mStr == NULL) { puts("!!!!!!!!!!!!!!!!!!!!!!!!!!"); }
        
         strcpy(self->mStr, str);
         self->mLen = len;
@@ -233,6 +238,7 @@ void string_put2(string_obj* self, char c)
         
         self->mMallocLen = 16;
         self->mStr = (char*)MALLOC(sizeof(char)*self->mMallocLen);
+if(self->mStr == NULL) { puts("!!!!!!!!!!!!!!!!!!!!!!!!!!"); }
         
         self->mStr[0] = c;
         self->mStr[1] = 0;
