@@ -7,6 +7,8 @@
 
 extern BOOL gKitutukiSigInt;  // CTRL-Cが押されたらTRUE
 
+void test();
+
 //////////////////////////////////////////////////////////////////////
 // ??????
 //////////////////////////////////////////////////////////////////////
@@ -88,6 +90,33 @@ void vector_add(vector_obj* self, void* item)
    self->mCount++;
 }
 
+void vector_add_test(vector_obj* self, void* item, void* t)
+{
+puts("AAA");
+typedef int (*Test)(void);
+    Test test = t;
+
+   if(self->mCount == self->mTableSize) {
+puts("BBB");
+test();
+      self->mTableSize = self->mTableSize * 2;
+puts("BBB");
+test();
+
+      self->mTable = (void**)REALLOC(self->mTable
+                                , sizeof(void*)*self->mTableSize);
+puts("BBB");
+test();
+   }
+
+puts("CCC");
+test();
+   self->mTable[self->mCount] = item;
+   self->mCount++;
+puts("CCC");
+test();
+}
+
 //////////////////////////////////////////////////////////////////////
 // ?A?C?e????n?̈ʒu?ɑ}??
 //////////////////////////////////////////////////////////////////////
