@@ -545,7 +545,8 @@ static BOOL statment_tree_make_argv(sCommand* command, vector_obj* argv, vector_
 
             /// @@のコマンド展開 ///
             case 4: {
-                sStatments* statments = arg->mBody;
+                sAtCommand* at_command = arg->mBody;
+                sStatments* statments = at_command->mStatments;
                 sWFd* pipeout = sWFd_new(-1);
                 int rcode = run(statments, "quick command expansion"
                     , pipeout, pipein, 2, return_, enable_return, break_, enable_break, parent_blocks, FALSE
@@ -558,7 +559,7 @@ static BOOL statment_tree_make_argv(sCommand* command, vector_obj* argv, vector_
                 sRFd* tmp = sRFd_new2(-1, pipeout->mBuffer);
                 while(1) {
                     string_obj* str = STRING_NEW("");
-                    int ret = sRFd_read_oneline(tmp, str, gLineField);
+                    int ret = sRFd_read_oneline(tmp, str, at_command->mLineField);
 
                     if(ret < -1) {
                         return FALSE;
@@ -1080,7 +1081,7 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
                 vector_add(gStackFrameRefs, HASH_NEW(30));
 
                 /// 引数を渡す ///
-                hash_obj* fun_argv_before = hash_item(object2->mArrays, "ARGV");
+                hash_obj* fun_argv_before = hash_item(gArrays, "ARGV");
 
                 int k;
                 vector_obj* argv2 = VECTOR_NEW(30);
@@ -1089,7 +1090,7 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
                     vector_add(argv2, STRING_NEW(value));
                 }
                 
-                hash_put(object2->mArrays, "ARGV", argv2);
+                hash_put(gArrays, "ARGV", argv2);
 
                 *rcode = run(klass->statments, title, nextout
                                 , nextin, nexterr
@@ -1104,12 +1105,12 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
                 vector_pop_back(gStackFrameHashs);
                 vector_pop_back(gStackFrameRefs);
 
+                /// 引数を戻す
+                hash_put(gArrays, "ARGV", fun_argv_before);
+
                 /// スタックフレームの記録を消す
                 string_obj* sname_strace = vector_pop_back(gStackTraceFName);
                 int line_strace = (int)vector_pop_back(gStackTraceLineNum);
-                /// 引数を戻す
-                hash_put(object2->mArrays, "ARGV", fun_argv_before);
-
                 if(*rcode < 0) {
                     string_push_back(gErrMsg, string_c_str(sname_strace));
                     char tmp[1024];
@@ -1140,7 +1141,7 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
         else if(method_class) {
             if(vector_size(blocks) == 1) {
                 /// 引数を渡す ///
-                hash_obj* fun_argv_before = hash_item(object2->mArrays, "ARGV");
+                hash_obj* fun_argv_before = hash_item(gArrays, "ARGV");
 
                 int k;
                 vector_obj* argv2 = VECTOR_NEW(30);
@@ -1149,7 +1150,7 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
                     vector_add(argv2, STRING_NEW(value));
                 }
                 
-                hash_put(object2->mArrays, "ARGV", argv2);
+                hash_put(gArrays, "ARGV", argv2);
 
                 /// スタックフレームの記録を付ける
                 vector_add(gStackTraceFName, STRING_NEW(sname));
@@ -1178,13 +1179,13 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
                 vector_pop_back(gStackFrameHashs);
                 vector_pop_back(gStackFrameRefs);
 
+                /// 引数を戻す
+                hash_put(gArrays, "ARGV", fun_argv_before);
+
                 /// スタックフレームの記録を消す
                 string_obj* sname_strace = vector_pop_back(gStackTraceFName);
                 int line_strace = (int)vector_pop_back(gStackTraceLineNum);
 
-                /// 引数を戻す
-                hash_put(object2->mArrays, "ARGV", fun_argv_before);
-
                 if(*rcode < 0) {
                     string_push_back(gErrMsg, string_c_str(sname_strace));
                     char tmp[1024];
@@ -1221,7 +1222,7 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
                 vector_add(gStackFrameRefs, HASH_NEW(30));
 
                 /// 引数を渡す ///
-                hash_obj* fun_argv_before = hash_item(object2->mArrays, "ARGV");
+                hash_obj* fun_argv_before = hash_item(gArrays, "ARGV");
 
                 int k;
                 vector_obj* argv2 = VECTOR_NEW(30);
@@ -1230,7 +1231,7 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
                     vector_add(argv2, STRING_NEW(value));
                 }
                 
-                hash_put(object2->mArrays, "ARGV", argv2);
+                hash_put(gArrays, "ARGV", argv2);
 
                 if(object) {
                     klass = hash_item(object->mClasses, klassname);
@@ -1264,7 +1265,7 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
                 *break_ = FALSE;
 
                 /// 引数を戻す
-                hash_put(object2->mArrays, "ARGV", fun_argv_before);
+                hash_put(gArrays, "ARGV", fun_argv_before);
 
                 // スタックを埋める
                 vector_pop_back(gStackFrame);
@@ -1319,7 +1320,7 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
             vector_obj* fun_argv;
             if(strcmp(string_c_str(method->arg_name), "") == 0) {
                 /// 前の引数を保存 ///
-                fun_argv_before = hash_item(object2->mArrays, "ARGV");
+                fun_argv_before = hash_item(gArrays, "ARGV");
 
                 /// 引数を渡す ///
                 int k;
@@ -1329,7 +1330,7 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
                     vector_add(fun_argv, STRING_NEW(value));
                 }
                 
-                hash_put(object2->mArrays, "ARGV", fun_argv);
+                hash_put(gArrays, "ARGV", fun_argv);
 
                 vector_add(gStackFrame, HASH_NEW(30));
                 vector_add(gStackFrameHashs, HASH_NEW(30));
@@ -1372,11 +1373,6 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
                 }
             }
 
-            /// スタックフレームの記録を消す
-            string_obj* sname_strace = vector_pop_back(gStackTraceFName);
-            int line_strace = (int)vector_pop_back(gStackTraceLineNum);
-
-
             if(strcmp(string_c_str(method->arg_name), "") == 0) {
                 /// スタックを消す ///
                 vector_pop_back(gStackFrame);
@@ -1386,11 +1382,11 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
                 vector_pop_back(gStackFrameRefs);
 
                 /// 引数を消す ///
-                hash_erase(object2->mArrays, "ARGV");
+                hash_erase(gArrays, "ARGV");
 
                 /// 前の引数を元に戻す ///
                 if(fun_argv_before) {
-                    hash_put(object2->mArrays, "ARGV", fun_argv_before);
+                    hash_put(gArrays, "ARGV", fun_argv_before);
                 }
             }
             else {
@@ -1404,6 +1400,10 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
                 }
             }
 
+            /// スタックフレームの記録を消す
+            string_obj* sname_strace = vector_pop_back(gStackTraceFName);
+            int line_strace = (int)vector_pop_back(gStackTraceLineNum);
+
             if(*rcode == -1) {
                 string_push_back(gErrMsg, string_c_str(sname_strace));
                 char tmp[1024];
@@ -1495,11 +1495,6 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
                 }
             }
 
-            /// スタックフレームの記録を消す
-            string_obj* sname_strace = vector_pop_back(gStackTraceFName);
-            int line_strace = (int)vector_pop_back(gStackTraceLineNum);
-
-
             if(strcmp(string_c_str(fun->arg_name), "") == 0) {
                 /// スタックを消す ///
                 vector_pop_back(gStackFrame);
@@ -1527,6 +1522,10 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
                 }
             }
 
+            /// スタックフレームの記録を消す
+            string_obj* sname_strace = vector_pop_back(gStackTraceFName);
+            int line_strace = (int)vector_pop_back(gStackTraceLineNum);
+
             if(*rcode == -1) {
                 string_push_back(gErrMsg, string_c_str(sname_strace));
                 char tmp[1024];
@@ -1572,7 +1571,7 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
 
             BOOL r = inner_fun->mFun(rcode, argv, blocks, parent_blocks
                         , nextout, nextin, nexterr
-                        , title, input);
+                        , title, input, sname, sline);
 
             /// 終了コード ///
             if(statment->mRCodeReverse) {
